'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const turboQuery = require('turbo-query');
const vue = require('vue');

const injectionKey = Symbol("vue-unquery-context");
function injectQueryOptions(value) {
  return vue.inject(injectionKey, value);
}
async function createQueryResource(key, options) {
  const contextOptions = injectQueryOptions();
  const turboQuery$1 = options?.turbo?.query ?? contextOptions?.turbo?.query ?? turboQuery.query;
  const turboMutate = options?.turbo?.mutate ?? contextOptions?.turbo?.mutate ?? turboQuery.mutate;
  const turboSubscribe = options?.turbo?.subscribe ?? contextOptions?.turbo?.subscribe ?? turboQuery.subscribe;
  const turboForget = options?.turbo?.forget ?? contextOptions?.turbo?.forget ?? turboQuery.forget;
  const turboAbort = options?.turbo?.abort ?? contextOptions?.turbo?.abort ?? turboQuery.abort;
  const computedKey = vue.computed(() => {
    try {
      return key();
    } catch {
      return null;
    }
  });
  const resource = vue.ref();
  const error = vue.ref();
  const isRefetching = vue.ref(false);
  vue.watch(error, (e) => {
    if (e !== void 0)
      throw e;
  });
  if (computedKey.value) {
    resource.value = await turboQuery$1(computedKey.value, {
      stale: true,
      ...options
    });
  }
  async function refetch(opts) {
    if (!computedKey.value)
      return;
    return await turboQuery$1(computedKey.value, {
      stale: false,
      ...options,
      ...opts
    });
  }
  function localMutate(item) {
    if (!computedKey.value)
      return;
    turboMutate(computedKey.value, item);
  }
  function localForget() {
    if (!computedKey.value)
      return;
    turboForget(computedKey.value);
  }
  function localAbort(reason) {
    if (!computedKey.value)
      return;
    turboAbort(computedKey.value, reason);
  }
  const unsubscribe = vue.watch(computedKey, async (key2, _old, onCleanup) => {
    isRefetching.value = false;
    if (!key2)
      return;
    const unsubscribeMutate = turboSubscribe(key2, "mutated", (item) => {
      resource.value = item;
    });
    const unsubscribeRefetching = turboSubscribe(key2, "refetching", () => {
      isRefetching.value = true;
    });
    const unsubscribeResolved = turboSubscribe(key2, "resolved", (item) => {
      isRefetching.value = false;
      resource.value = item;
    });
    const unsubscribeErrors = turboSubscribe(key2, "error", (e) => {
      isRefetching.value = false;
      error.value = e;
    });
    resource.value = await turboQuery$1(key2, {
      stale: true,
      ...options
    });
    onCleanup(() => {
      unsubscribeMutate();
      unsubscribeRefetching();
      unsubscribeResolved();
      unsubscribeErrors();
    });
  }, { immediate: true });
  if (vue.getCurrentInstance())
    vue.onUnmounted(() => unsubscribe());
  return [
    resource,
    {
      refetch,
      mutate: localMutate,
      forget: localForget,
      abort: localAbort,
      unsubscribe,
      error: vue.readonly(error),
      isRefetching: vue.readonly(isRefetching)
    }
  ];
}
function useQuery(key, options) {
  const contextOptions = injectQueryOptions();
  const turboQuery$1 = options?.turbo?.query ?? contextOptions?.turbo?.query ?? turboQuery.query;
  const turboMutate = options?.turbo?.mutate ?? contextOptions?.turbo?.mutate ?? turboQuery.mutate;
  const turboSubscribe = options?.turbo?.subscribe ?? contextOptions?.turbo?.subscribe ?? turboQuery.subscribe;
  const turboForget = options?.turbo?.forget ?? contextOptions?.turbo?.forget ?? turboQuery.forget;
  const turboAbort = options?.turbo?.abort ?? contextOptions?.turbo?.abort ?? turboQuery.abort;
  const immediate = options?.immediate ?? contextOptions?.immediate ?? true;
  const computedKey = vue.computed(() => {
    try {
      return key();
    } catch {
      return null;
    }
  });
  const resource = vue.ref();
  const error = vue.ref();
  const isRefetching = vue.ref(false);
  (async () => {
    if (!computedKey.value || !immediate)
      return;
    resource.value = await turboQuery$1(computedKey.value, {
      stale: true,
      ...options
    });
  })();
  async function refetch(opts) {
    if (!computedKey.value)
      return;
    return await turboQuery$1(computedKey.value, {
      stale: false,
      ...options,
      ...opts
    });
  }
  function localMutate(item) {
    if (!computedKey.value)
      return;
    turboMutate(computedKey.value, item);
  }
  function localForget() {
    if (!computedKey.value)
      return;
    turboForget(computedKey.value);
  }
  function localAbort(reason) {
    if (!computedKey.value)
      return;
    turboAbort(computedKey.value, reason);
  }
  const unsubscribe = vue.watch(computedKey, async (key2, _old, onCleanup) => {
    isRefetching.value = false;
    if (!key2)
      return;
    const unsubscribeMutate = turboSubscribe(key2, "mutated", (item) => {
      resource.value = item;
    });
    const unsubscribeRefetching = turboSubscribe(key2, "refetching", () => {
      isRefetching.value = true;
    });
    const unsubscribeResolved = turboSubscribe(key2, "resolved", (item) => {
      isRefetching.value = false;
      resource.value = item;
    });
    const unsubscribeErrors = turboSubscribe(key2, "error", (e) => {
      isRefetching.value = false;
      error.value = e;
    });
    resource.value = await turboQuery$1(key2, {
      stale: true,
      ...options
    });
    onCleanup(() => {
      unsubscribeMutate();
      unsubscribeRefetching();
      unsubscribeResolved();
      unsubscribeErrors();
    });
  }, { immediate });
  if (vue.getCurrentInstance())
    vue.onUnmounted(() => unsubscribe());
  return [
    resource,
    {
      refetch,
      mutate: localMutate,
      forget: localForget,
      abort: localAbort,
      unsubscribe,
      error: vue.readonly(error),
      isRefetching: vue.readonly(isRefetching)
    }
  ];
}

exports.createQueryResource = createQueryResource;
exports.injectQueryOptions = injectQueryOptions;
exports.injectionKey = injectionKey;
exports.useQuery = useQuery;
