import { query, mutate, subscribe, forget, abort } from 'turbo-query';
import { inject, computed, ref, watch, getCurrentInstance, onUnmounted, readonly } from 'vue';

const injectionKey = Symbol("vue-unquery-context");
function injectQueryOptions(value) {
  return inject(injectionKey, value);
}
async function createQueryResource(key, options) {
  const contextOptions = injectQueryOptions();
  const turboQuery = options?.turbo?.query ?? contextOptions?.turbo?.query ?? query;
  const turboMutate = options?.turbo?.mutate ?? contextOptions?.turbo?.mutate ?? mutate;
  const turboSubscribe = options?.turbo?.subscribe ?? contextOptions?.turbo?.subscribe ?? subscribe;
  const turboForget = options?.turbo?.forget ?? contextOptions?.turbo?.forget ?? forget;
  const turboAbort = options?.turbo?.abort ?? contextOptions?.turbo?.abort ?? abort;
  const computedKey = computed(() => {
    try {
      return key();
    } catch {
      return null;
    }
  });
  const resource = ref();
  const error = ref();
  const isRefetching = ref(false);
  watch(error, (e) => {
    if (e !== void 0)
      throw e;
  });
  if (computedKey.value) {
    resource.value = await turboQuery(computedKey.value, {
      stale: true,
      ...options
    });
  }
  async function refetch(opts) {
    if (!computedKey.value)
      return;
    return await turboQuery(computedKey.value, {
      stale: false,
      ...options,
      ...opts
    });
  }
  function localMutate(item) {
    if (!computedKey.value)
      return;
    turboMutate(computedKey.value, item);
  }
  function localForget() {
    if (!computedKey.value)
      return;
    turboForget(computedKey.value);
  }
  function localAbort(reason) {
    if (!computedKey.value)
      return;
    turboAbort(computedKey.value, reason);
  }
  const unsubscribe = watch(computedKey, async (key2, _old, onCleanup) => {
    isRefetching.value = false;
    if (!key2)
      return;
    const unsubscribeMutate = turboSubscribe(key2, "mutated", (item) => {
      resource.value = item;
    });
    const unsubscribeRefetching = turboSubscribe(key2, "refetching", () => {
      isRefetching.value = true;
    });
    const unsubscribeResolved = turboSubscribe(key2, "resolved", (item) => {
      isRefetching.value = false;
      resource.value = item;
    });
    const unsubscribeErrors = turboSubscribe(key2, "error", (e) => {
      isRefetching.value = false;
      error.value = e;
    });
    resource.value = await turboQuery(key2, {
      stale: true,
      ...options
    });
    onCleanup(() => {
      unsubscribeMutate();
      unsubscribeRefetching();
      unsubscribeResolved();
      unsubscribeErrors();
    });
  }, { immediate: true });
  if (getCurrentInstance())
    onUnmounted(() => unsubscribe());
  return [
    resource,
    {
      refetch,
      mutate: localMutate,
      forget: localForget,
      abort: localAbort,
      unsubscribe,
      error: readonly(error),
      isRefetching: readonly(isRefetching)
    }
  ];
}
function useQuery(key, options) {
  const contextOptions = injectQueryOptions();
  const turboQuery = options?.turbo?.query ?? contextOptions?.turbo?.query ?? query;
  const turboMutate = options?.turbo?.mutate ?? contextOptions?.turbo?.mutate ?? mutate;
  const turboSubscribe = options?.turbo?.subscribe ?? contextOptions?.turbo?.subscribe ?? subscribe;
  const turboForget = options?.turbo?.forget ?? contextOptions?.turbo?.forget ?? forget;
  const turboAbort = options?.turbo?.abort ?? contextOptions?.turbo?.abort ?? abort;
  const immediate = options?.immediate ?? contextOptions?.immediate ?? true;
  const computedKey = computed(() => {
    try {
      return key();
    } catch {
      return null;
    }
  });
  const resource = ref();
  const error = ref();
  const isRefetching = ref(false);
  (async () => {
    if (!computedKey.value || !immediate)
      return;
    resource.value = await turboQuery(computedKey.value, {
      stale: true,
      ...options
    });
  })();
  async function refetch(opts) {
    if (!computedKey.value)
      return;
    return await turboQuery(computedKey.value, {
      stale: false,
      ...options,
      ...opts
    });
  }
  function localMutate(item) {
    if (!computedKey.value)
      return;
    turboMutate(computedKey.value, item);
  }
  function localForget() {
    if (!computedKey.value)
      return;
    turboForget(computedKey.value);
  }
  function localAbort(reason) {
    if (!computedKey.value)
      return;
    turboAbort(computedKey.value, reason);
  }
  const unsubscribe = watch(computedKey, async (key2, _old, onCleanup) => {
    isRefetching.value = false;
    if (!key2)
      return;
    const unsubscribeMutate = turboSubscribe(key2, "mutated", (item) => {
      resource.value = item;
    });
    const unsubscribeRefetching = turboSubscribe(key2, "refetching", () => {
      isRefetching.value = true;
    });
    const unsubscribeResolved = turboSubscribe(key2, "resolved", (item) => {
      isRefetching.value = false;
      resource.value = item;
    });
    const unsubscribeErrors = turboSubscribe(key2, "error", (e) => {
      isRefetching.value = false;
      error.value = e;
    });
    resource.value = await turboQuery(key2, {
      stale: true,
      ...options
    });
    onCleanup(() => {
      unsubscribeMutate();
      unsubscribeRefetching();
      unsubscribeResolved();
      unsubscribeErrors();
    });
  }, { immediate });
  if (getCurrentInstance())
    onUnmounted(() => unsubscribe());
  return [
    resource,
    {
      refetch,
      mutate: localMutate,
      forget: localForget,
      abort: localAbort,
      unsubscribe,
      error: readonly(error),
      isRefetching: readonly(isRefetching)
    }
  ];
}

export { createQueryResource, injectQueryOptions, injectionKey, useQuery };
